// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_TECTONIC_OBSTACLESOLVER_HH
#define DUNE_TECTONIC_OBSTACLESOLVER_HH

#include "spatial-solving/tnnmg/localproblem.hh"


/**
 * \brief A local solver for quadratic obstacle problems
 *
 * \todo Add concept check for the function interface
 */
class ObstacleSolver
{
public:
  template<class Vector, class Functional, class BitVector>
  constexpr void operator()(Vector& x, const Functional& f, const BitVector& ignore) const
  {
      auto& A = f.quadraticPart();
      auto& r = f.linearPart();

      LocalProblem<std::decay_t<decltype(A)> , std::decay_t<decltype(r)>> localProblem(A, r, ignore);
      Vector newR;
      localProblem.getLocalRhs(x, newR);

      auto directSolver = Dune::Solvers::UMFPackSolver<std::decay_t<decltype(A)>, std::decay_t<decltype(r)>>();

      directSolver->setProblem(localProblem.getMat(), x, newR);
      directSolver->preprocess();
      directSolver->solve();

      const auto& lower = f.lowerObstacle();
      const auto& upper = f.upperObstacle();

      for (size_t i=0; i<x.size(); i++) {
        if (ignore[i][0])
            continue;

        if (x[i] < lower[i])
            x[i] = lower[i];
        if (x[i] > upper[i])
            x[i] = upper[i];
      }
  }
};

#endif
