#ifndef LEVEL_GLOBAL_PRECONDITIONER_HH
#define LEVEL_GLOBAL_PRECONDITIONER_HH

#include <string>

#include <dune/common/timer.hh>
#include <dune/common/fvector.hh>

#include <dune/solvers/iterationsteps/lineariterationstep.hh>

//#include <dune/istl/superlu.hh>
#include <dune/istl/umfpack.hh>

#include "../../data-structures/levelcontactnetwork.hh"


template <class LevelContactNetwork, class Solver, class MatrixType, class VectorType>
class LevelGlobalPreconditioner : public LinearIterationStep<MatrixType, VectorType> {

private:
    using Base = LinearIterationStep<MatrixType, VectorType>;

    using ctype = typename LevelContactNetwork::ctype;

    const LevelContactNetwork& levelContactNetwork_;
    const int level_;

    std::shared_ptr<Solver> solver_;

public:

    LevelGlobalPreconditioner(const LevelContactNetwork& levelContactNetwork, const int level = 0) :
          levelContactNetwork_(levelContactNetwork),
          level_(level) {

        this->verbosity_ = QUIET;
    }

    void setSolver(Solver&& patchSolver) {
        solver_ = Dune::Solvers::wrap_own_share<PatchSolver>(std::forward<PatchSolver>(patchSolver));
    }

    void iterate() override {
        Dune::Timer timer;

        if (this->verbosity_ == FULL) {
            timer.reset();
            timer.start();
        }

        *(this->x_) = 0;

        auto& step = solver_->getIterationStep();
        step.setProblem(this->mat_, this->x_, this->rhs_);
        step.setIgnore(this->ignoreNodes_);

        solver_->check();
        solver_->preprocess();
        solver_->solve();

        *(this->x_) = solver_->getSol();

        if (this->verbosity_ == FULL) {
            std::cout << "LevelGlobalPreconditioner::iterate() Solving global problem took: " << timer.stop() << " seconds" << std::endl;
        }
    }

    /*void iterate() override {
        Dune::Timer timer;
        timer.reset();
        timer.start();

        this->x_->resize(matrix_.M());
        *(this->x_) = 0;

        #if HAVE_SUPERLU
        Dune::InverseOperatorResult res;
        VectorType rhsCopy(rhs_);

        Dune::UMFPack<MatrixType> solver(matrix_);
        solver.apply(*(this->x_), rhsCopy, res);

        #else
        #error No SuperLU!
        #endif

        //std::cout << "LevelGlobalPreconditioner::iterate() Solving global problem took: " << timer.elapsed() << " seconds" << std::endl;

    }*/

};
#endif

