#ifndef OSC_LOCAL_PROBLEM_HH
#define OSC_LOCAL_PROBLEM_HH

#include <math.h>   
#include <dune/common/fmatrix.hh>
#include <dune/common/function.hh>
#include <dune/common/timer.hh>

#include <dune/istl/matrixindexset.hh>
//#include <dune/istl/superlu.hh>
#include <dune/istl/umfpack.hh>

#include <dune/fufem/assemblers/localoperatorassembler.hh>

#include "../../utils/debugutils.hh"

template <class MatrixType, class DomainType, class RangeType = DomainType>
class LocalProblem {
  
private:    
    typedef typename MatrixType::block_type block_type;
    typedef typename MatrixType::field_type ctype;

    const static size_t dim = DomainType::block_type::dimension;

    using BitVector = Dune::BitSetVector<dim>;

    MatrixType localMat;
    const RangeType& rhs_;
    const BitVector& ignoreNodes_;

   /* size_t flatIndex(const size_t blockIdx, const size_t localBlockIdx) {
        return blockIdx*dim + localBlockIdx;
    }

    bool isAllIgnored(const size_t blockIdx) {
        bool res = true;
        size_t flatIdx = blockIdx*dim;

        for (size_t d=0; d<dim; d++) {
            res = res && (globalToLocal_[flatIdx+d] == -1);
        }

        return res;
    }*/
    template <class LocalMat, class LocalBitVector>
    void computeLocalMat(LocalMat& localMat, const LocalMat& refMat, const LocalBitVector& localIgnore, bool isDiagonal=false) {
        if (isDiagonal) {
            for (size_t i=0; i<refMat.N(); i++) {
                bool isIgnored = localIgnore[i];
                for (size_t j=0; j<refMat.M(); j++) {
                    if (isIgnored || refMat[i][j]==0)
                        localMat[i][j] = (double) (i==j);
                    else
                        localMat[i][j] = refMat[i][j];
                }
            }
        } else {
            for (size_t i=0; i<refMat.N(); i++) {
                bool isIgnored = localIgnore[i];
                for (size_t j=0; j<refMat.M(); j++) {
                    if (isIgnored)
                        localMat[i][j] = 0;
                    else
                        localMat[i][j] = refMat[i][j];
                }
            }
        }
    }

public:
    LocalProblem(const MatrixType& mat,
                 const RangeType& rhs,
                 const BitVector& ignoreNodes) :
      rhs_(rhs),
      ignoreNodes_(ignoreNodes)
	{
	  
	// construct globalToLocal map
       /* std::vector<int, int> localToGlobal;
        std::vector<int, int> globalToLocal(ignoreNodes.size()*dim, -1);
        int localIdx = 0;
        for (size_t i=0; i<ignoreNodes.size(); ++i) {
            const auto& ignoreNode = ignoreNodes[i];
            for (size_t d=0; d<dim; d++) {
                if (not toBool(ignoreNode[d])) {
                    size_t flatIdx = flatIndex(i, d);
                    localToGlobal[localIdx] = flatIdx;
                    globalToLocal[flatIdx] = localIdx;
                    localIdx++;
                }
            }
        }*/

	// build local stiffness matrix
        localMat = mat;
	
        for(size_t rowIdx = 0; rowIdx<localMat.N(); rowIdx++) {
            const auto& row = mat[rowIdx];
	    
            auto colIt = row.begin();
            const auto& colEndIt = row.end();
            for(; colIt!=colEndIt; ++colIt) {
                const auto colIdx = colIt.index();
                computeLocalMat(localMat[rowIdx][colIdx], row[colIdx], ignoreNodes_[rowIdx], rowIdx==colIdx);
            }
        }   
    }

    MatrixType& getMat() {
	return localMat;
    }
    
    void getLocalRhs(const DomainType& iterate, RangeType& newRhs) {
        newRhs = rhs_;

        for (size_t i=0; i<newRhs.size(); i++) {
            for (size_t d=0; d<dim; d++) {
                if (ignoreNodes_[i][d]) {
                    newRhs[i][d] = iterate[i][d];
                }
            }
        }
    }

  /*  void solve(DomainType& x){
        #if HAVE_SUPERLU
            RangeType localRhsCopy(localRhs);
            Dune::InverseOperatorResult res;

            x.resize(localMat.M());

            Dune::UMFPack<MatrixType> directSolver(localMat);
            directSolver.apply(x, localRhsCopy, res);
        #else
        #error No SuperLU!
        #endif
    }*/
};

#endif
